#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import pathlib
import meshio
from datetime import datetime
from HM.source.assemble_2D import assemble_k, assemble_f, calculate_point_strains
from HM.source.util_2D import *

def main():
    '''
    # Main routine for running the pipe problem'

    Variables to change are found under Define problem
    Remember to scale E-mod in line 45 accordingly
    :return: writes .vtu files and .txt files with the results in the results_path
    '''
    # Defining pathing
    paths = {'wdir': pathlib.Path('/Users/maren/MASTER/HM/')}
    paths['data_path'] = paths['wdir'] / 'data'
    paths['geo_path'] = paths['data_path'] / 'geo_files'
    paths['mesh_path'] = paths['data_path'] / 'mesh_files'
    paths['result_path'] = paths['data_path'] / 'result_files'

    # Define problem
    meshes = ['ANS_1_']
    problem = 'pipe_2D_'

    dofs = (0, 1)
    elemtype = 1  #0 = Trad, ANS = 1
    destination = paths['result_path'] / 'CST_disp'

    num_dofs = []
    tip_defs = []

    emat = np.array([200000, 0.29])
    num_steps = 10
    num_iter= 5
    m = 2 #SH strain

    for mesh_size in meshes:

        # Reading mesh data
        file = problem + mesh_size
        meshfile = paths['mesh_path'] / (file + '.msh')

        msh = meshio.read(filename=meshfile)

        ndofs_per_node = 2
        num_dof = msh.points.shape[0] * ndofs_per_node

        for step in range(num_steps):
            print('======== Step {} ==============='.format(step))

            tot_disp = np.zeros(num_dof)

            for iter in range(num_iter):
                print('==== Step = {}  Iter = {}'.format(step, iter))
                
                # Assemble matrix equation
                k_sys = assemble_k(mesh=msh, setname='domain', elemtype=elemtype, emat=emat, dispvec=tot_disp, m=m)

                # add external load:
                load_factor = (step + 1) / num_steps

                load_vecY = np.zeros(k_sys.shape[0])
                load_vecX = np.zeros(k_sys.shape[0])


                add_load(mesh=msh, setname='loadY', load_vec=load_vecY, load_magnitude=500*load_factor, dof=1)
                add_load(mesh=msh, setname='loadX', load_vec=load_vecX, load_magnitude=500*load_factor, dof=0)

                load_vec = load_vecY + load_vecX
                force_ext_norm = np.linalg.norm(load_vec)
                print('Force external norm: {}'.format(force_ext_norm))

                f_int = assemble_f(mesh=msh, setname='domain', elemtype=elemtype, emat=emat, dispvec=tot_disp, m=m)
                force_int_norm = np.linalg.norm(f_int)
                print('Force internal norm: {}'.format(force_int_norm))

                #Calculate force residual
                force_residual = load_vec - f_int

                set_fixed_dofs(mesh=msh, setname='fixedEnd', k_sys=k_sys, rhs=force_residual, dofs=dofs)

                residual_norm = np.linalg.norm(force_residual)
                print('Force residual norm: {}'.format(residual_norm))

                #Solve matrix equation
                inc_disp = np.linalg.solve(k_sys, force_residual)
                tot_disp += inc_disp

                inc_disp_norm = np.linalg.norm(inc_disp)
                print('incremental disp norm: {}'.format(inc_disp_norm))


                if ( residual_norm < 1.0e-6):
                    break

            #solve Green
            point_strain = calculate_point_strains(mesh=msh, setname='domain', elemtype=elemtype, emat=emat, tot_disp=tot_disp, m=m)

            # Format results
            tot_disp = tot_disp.reshape((msh.points.shape[0], 2))
            disp = tot_disp[:, 0:2]
            disp_3D = np.zeros((disp.shape[0], 3))
            disp_3D[:,0:2] = disp[:,0:2]

            msh.point_data['Displacements'] = disp_3D
            msh.point_data['Strain'] = point_strain

            # Write results to file for post-processing
            num_dofs.append(tot_disp.size)
            tip_defs.append(disp.max())
            meshio.write(filename=(destination / (file +  str(step+1) + '.vtu')), mesh=msh)

if __name__ == '__main__':
    main()