#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import pathlib
import meshio
from datetime import datetime
from HM.source.assemble_k_3D import assemble_k
from HM.source.util_3D import *

def main():
    '''
    # Main routine for running the pipe problem'

    Variables to change are found under Define problem
    Remember to scale E-mod in line 45 accordingly
    :return: writes .vtu files and .txt files with the results in the results_path
    '''
    # Defining pathing
    paths = {'wdir': pathlib.Path('/Users/maren/MASTER/HM/')}
    paths['data_path'] = paths['wdir'] / 'data'
    paths['geo_path'] = paths['data_path'] / 'geo_files'
    paths['mesh_path'] = paths['data_path'] / 'mesh_files'
    paths['result_path'] = paths['data_path'] / 'result_files'

    # Define problem
    meshes = ['1']
    problem = 'pipe_3D_'

    dofs = (0, 1, 2)  #(0, 1, 2, 3, 4, 5)
    elemtype = 0 #endret til csth
    destination = paths['result_path'] / 'CST_disp'

    num_dofs = []
    tip_defs = []

    for mesh_size in meshes:
        file = problem + mesh_size
        meshfile = paths['mesh_path'] / (file + '.msh')

        # Reading mesh data
        msh = meshio.read(filename=meshfile)

        # Assemble matrix equation
        # Settings from Haugens document, works good for H1
        k_sys = assemble_k(mesh=msh, setname='domain', elemtype=elemtype, emat=(30000 * 12, 0.25))

        load_vec = np.zeros(k_sys.shape[0])
        set_fixed_dofs(mesh=msh, setname='fixedEnd', k_sys=k_sys, dofs=dofs)
        add_load(mesh=msh, setname='dispEnd', load_vec=load_vec, load_magnitude=40, dof=1)

        # Solve matrix equation
        tot_disp = np.linalg.solve(k_sys, load_vec).reshape((msh.points.shape[0], 3))  #endret fra 6 til 3
        
        # Format results
        disp = tot_disp[:, 0:3]
        #disp_3D = np.zeros((disp.shape[0], 3))
        #disp_3D[:,0:2] = disp[:,0:3]

        msh.point_data['Displacements'] = disp

        
        
        
        # Write results to file for post-processing
        num_dofs.append(tot_disp.size)
        tip_defs.append(disp.max())
        meshio.write(filename=(destination / (file + '.vtu')), mesh=msh)

    tip_def_norm = (np.asarray(tip_defs) / np.max(tip_defs) * 100).round(2).tolist()
    res_summary = 'Created at {}\n' \
                  'Element type {} - 0 = CSTh, 1 = H1\n' \
                  'Fixed dofs: {}\n' \
                  'Mesh grids: {}\n' \
                  'Num dofs: {}\n' \
                  'Tip defs: {}\n' \
                  'Norm tip defs: {}\n \n' \
        .format(datetime.now(),
                elemtype,
                dofs,
                meshes,
                num_dofs,
                tip_defs,
                tip_def_norm
                )

    with open((destination / (problem + '.txt')), "a") as text_results:
        text_results.write(res_summary)


if __name__ == '__main__':
    main()