import HM.source.constitutive_2D as const
import HM.source.tri3_tangent_stiffness as tan
import HM.source.tri3_large_strains as tri3
import HM.source.util_2D as util
import HM.source.bar_large_strain as bar3
import HM.source.bar_element_tangent as bar
from HM.source.util_2D import *


def assemble_k(mesh, setname, elemtype, emat, dispvec, m):
    '''
    Assembles system stiffness matrix K_sys in dense form for given mesh with defined domain-set
    :param mesh:        MeshIO object generated from .msh file
    :param setname:     string, setname of the domain as defined in Gmsh e.g. 'domain'
    :param elemtype:    integer, 0 for CST traditional element, 1 for ANS
    :param emat:        tuple, (E-modulus, Poisson)
    :param m:           integer, Seth-hill strain
    :return:            np.array (dofs, dofs), dense-matrix of the system stiffness
    '''
    ndofs_per_node = 2
    num_dof  = mesh.points.shape[0] * ndofs_per_node
    elements = extract_set(mesh=mesh, setname=setname, connectivity=True)
    k_sys = np.zeros((num_dof, num_dof))

    for indx in range(elements.shape[0]):
        ex, ey = extract_element_coords(mesh=mesh, elements=elements, indx=indx)
        dof_indx = extract_dof_indx(elements=elements, indx=indx) 
        
        disp_element = dispvec[np.ix_(dof_indx)]
        if elemtype == 0:
            kt = tan.get_tangent_Stiffness_matrix_trad(ex, ey, disp_element, C = const.C_glob(emat[0], emat[1]), m=m)
        elif elemtype == 1:
            kt = bar.get_tangent_stiffness_ANS(ex,ey, disp_element, C = const.C_glob(emat[0], emat[1]), m=m)
        k_sys[np.ix_(dof_indx, dof_indx)] += kt

    return k_sys

def assemble_f(mesh, setname, elemtype, emat, dispvec, m):
    '''
    Assembles system stiffness matrix K_sys in dense form for given mesh with defined domain-set
    :param mesh:        MeshIO object generated from .msh file
    :param setname:     string, setname of the domain as defined in Gmsh e.g. 'domain'
    :param elemtype:    integer, 0 for CST traditional element, 1 for ANS
    :param emat:        tuple, (E-modulus, Poisson)
    :param m:           integer, Seth-hill strain
    :return:            np.array (dofs, dofs), dense-matrix of the system stiffness
    '''
    ndofs_per_node = 2
    num_dof  = mesh.points.shape[0] * ndofs_per_node
    elements = extract_set(mesh=mesh, setname=setname, connectivity=True)

    f_int_sys = np.zeros(num_dof)

    for indx in range(elements.shape[0]):
        ex, ey = extract_element_coords(mesh=mesh, elements=elements, indx=indx)
        
        dof_indx = extract_dof_indx(elements=elements, indx=indx)

        disp_element = dispvec[np.ix_(dof_indx)]
        if elemtype ==0:
            f_int = tan.get_internal_force_trad(ex, ey, disp_element, C =const.C_glob(emat[0], emat[1]), m=m)
        elif elemtype ==1:
            f_int = bar.get_internal_force_ANS(ex,ey,disp_element, C =const.C_glob(emat[0], emat[1]), m=m)
        f_int_sys[np.ix_(dof_indx)] += f_int

    return f_int_sys


def assemble_strain(mesh, setname, elemtype, emat, tot_disp, m):
    '''
    Assembles system stiffness matrix K_sys in dense form for given mesh with defined domain-set
    :param mesh:        MeshIO object generated from .msh file
    :param setname:     string, setname of the domain as defined in Gmsh e.g. 'domain'
    :param elemtype:    integer, 0 for CSTh traditional element, 1 for ANS
    :param emat:        tuple, (E-modulus, Poisson)
    :param m:           integer, Seth-hill strain
    :return:            np.array (dofs, dofs), dense-matrix of the system stiffness
    '''
    ndofs_per_node = 2
    num_dof = mesh.points.shape[0] * ndofs_per_node

    elements = extract_set(mesh=mesh, setname=setname, connectivity=True)
    num_elements = elements.shape[0]

    e_sys = np.zeros((num_elements, 3))

    for indx in range(num_elements):
        ex, ey = extract_element_coords(mesh=mesh, elements=elements, indx=indx)

        dof_indx = extract_dof_indx(elements=elements, indx=indx)
        disp_element = tot_disp[np.ix_(dof_indx)]

        if elemtype == 0:
            e_green = tri3.get_strainVec_trad(ex, ey, disp_element, m=m)
        elif elemtype ==1:
            e_green = bar3.get_CartesianCoordinates_strain(ex, ey, disp_element, m=m)

        e_sys[indx, :] = e_green[:]
    return e_sys


def calculate_point_strains( mesh, setname, elemtype, emat, tot_disp, m):
    """
    Assembles internal forces

    :param tot_disp:    total displacemenbt for all degrees of freedom
    :param mesh:        MeshIO object generated from .msh file
    :param setname:     string, setname of the domain as defined in Gmsh e.g. 'domain'
    :param elemtype:    integer, 0 for CSTh traditional element, 1 for ANS
    :param alphab:      float, scaling factor for rotational contribution to basic stiffness
    :param alphat:      float, scaling factor for torsion strain
    :param beta:        float, scaling factor for higher order stiffness
    :param fb:          float, scaling factor for basic stiffness
    :param m:           integer, Seth-hill strain
    :return:            np.array (dofs, dofs), dense-matrix of the system stiffness
    """

    element_strains = assemble_strain(mesh, setname, elemtype, emat, tot_disp, m=m)

    num_points = mesh.points.shape[0]
    point_strains = np.zeros((num_points,3))
    num_added_to_point = np.zeros(num_points)

    elements = util.extract_set(mesh=mesh, setname=setname, connectivity=True)

    for idx in range(elements.shape[0]):
        elem = elements[idx]
        i, j, k, = elem[0], elem[1], elem[2]
        point_strains[i,:] += element_strains[idx,:]
        point_strains[j,:] += element_strains[idx,:]
        point_strains[k,:] += element_strains[idx,:]
        num_added_to_point[i] += 1
        num_added_to_point[j] += 1
        num_added_to_point[k] += 1

    for idx in range(point_strains.shape[0]):
        if (num_added_to_point[idx] > 0):
            point_strains[idx,:] /= num_added_to_point[idx]

    return point_strains

