import numpy as np
import scipy.sparse as sp
from HM.source.util_3D import *
from HM.source.ke_4tet_3D import ke_4tet


def assemble_k(mesh, setname, elemtype, emat):
    '''
    Assembles system stiffness matrix K_sys in dense form for given mesh with defined domain-set


    :param mesh:        MeshIO object generated from .msh file
    :param setname:     string, setname of the domain as defined in Gmsh e.g. 'domain'
    :param elemtype:    integer, 0 for CSTh element, 1 for ANDES
    :param emat:        tuple, (E-modulus, Poisson)
    :param alphab:      float, scaling factor for rotational contribution to basic stiffness
    :param alphat:      float, scaling factor for torsion strain
    :param beta:        float, scaling factor for higher order stiffness
    :param fb:          float, scaling factor for basic stiffness
    :return:            np.array (dofs, dofs), dense-matrix of the system stiffness
    '''
    ndofs_per_node = 3     #endre
    num_dof = mesh.points.shape[0] * ndofs_per_node
    elements = extract_set(mesh=mesh, setname=setname, connectivity=True)

    k_sys = np.zeros((num_dof, num_dof))

    for indx in range(elements.shape[0]):
        ex, ey, ez = extract_element_coords(mesh=mesh, elements=elements, indx=indx)

        ke = ke_4tet(ex, ey, ez, emat)
        dof_indx = extract_dof_indx(elements=elements, indx=indx)

        k_sys[np.ix_(dof_indx, dof_indx)] += ke

    return k_sys


def assemble_sparse(mesh, domain_set, bc_set, dofs, elemtype, emat, alphab, alphat, beta, fb):
    '''
    Creates the constrained system stiffness matrix K_sys on sparse-form, given constraints
    Requiring less memory allocation and allowing larger meshes

    Constructing a matrix using ijv format
    row  = np.array([0, 3, 1, 0])
    col  = np.array([0, 3, 1, 2])
    data = np.array([4, 5, 7, 9])
    coo_matrix((data, (row, col)), shape=(4, 4)).toarray()
    array([[4, 0, 9, 0],
           [0, 7, 0, 0],
           [0, 0, 0, 0],
           [0, 0, 0, 5]])

    :param mesh:            MeshIO object generated from .msh file
    :param domain_set:      string, setname of the domain as defined in Gmsh e.g. 'domain'
    :param bc_set:          string, setname of the boundary as defined in Gmsh e.g. 'fixedEnd'
    :param dofs:            tuple, constrained dofs in the nodes of bc_set
    :param elemtype:        integer, 0 for CSTh element, 1 for ANDES
    :param emat:            tuple, (E-modulus, Poisson)
    :param alphab:          float, scaling factor for rotational contribution to basic stiffness
    :param alphat:          float, scaling factor for torsion strain
    :param beta:            float, scaling factor for higher order stiffness
    :param fb:              float, scaling factor for basic stiffness
    :return:                csr-matrix, compressed row-matrix of the system stiffness
    '''
    elements = extract_set(mesh=mesh, setname=domain_set, connectivity=True)
    bc_indx = nodal_IDs_to_dof_indx(nodeset=extract_set(mesh, bc_set, False), dofs=dofs)
    rows = bc_indx[:]
    cols = bc_indx[:]
    stiff_comps = [1 for _ in range(rows.__len__())]

    for elem_indx in range(elements.shape[0]):
        ex, ey, ez = extract_element_coords(mesh=mesh, elements=elements, indx=elem_indx)
        elem_dof = extract_dof_indx(elements=elements, indx=elem_indx)
        ke = ke_4tet(ex, ey, ez, emat, alphab, alphat, beta, fb, elemtype)
        pop_indx = []

        for indx in range(elem_dof.shape[0]):
            if elem_dof.data[indx] in bc_indx:
                pop_indx.append(indx)

        ke = np.delete(ke, pop_indx, 0)
        ke = np.delete(ke, pop_indx, 1)
        elem_dof = np.delete(elem_dof, pop_indx)

        lenght = elem_dof.shape[0]
        elem_rows = np.empty(lenght ** 2, dtype=int)
        elem_cols = np.empty(lenght ** 2, dtype=int)
        for i in range(lenght):
            elem_rows[lenght * i:(lenght * i) + lenght:] = elem_dof.data[i]
            elem_cols[lenght * i:(lenght * i) + lenght:] = elem_dof

        rows.extend(elem_rows.tolist())
        cols.extend(elem_cols.tolist())
        stiff_comps.extend(ke.flatten().tolist())

    k_sys = sp.coo_matrix((stiff_comps, (rows, cols))).tocsr()

    return k_sys
