import HM.source.bar_large_strain as bar
import numpy as np

#-------------------------Internal-force---------------------------

def get_internal_force_ANS(ex,ey, dispVec, C, m):
    
    B_lv  = get_B_lv_matrix(ex,ey, dispVec)
    B_el  = get_B_el_matrix(ex,ey, dispVec, m)

    eps_n = bar.get_NaturalCoordinates_strain(ex, ey, dispVec, m)
    
    T_cn  = bar.get_Triangle_element_transformation_matrix(ex, ey, dispVec)
    T_nc  = np.linalg.inv(T_cn)
    C_n   = T_nc.T @ C @ T_nc
    
    s     = C_n @ eps_n
    
    f_int = B_lv.T @ B_el.T @ s
    
    return f_int

#----------------------------Tangent-stiffness------------------------------------

def get_B_lv_matrix(ex,ey, dispVec):
    """ 
    B-matrix consisting of partial derivative of length as a function of 
    global displacements, unit vectors along side edge lengths 
    Returns B_lv = partial L / partial V
    """
    Ld = bar.get_length_deformed(ex, ey, dispVec)
    
    L1_u =  float(( ex[1] + dispVec[2] - ex[0] - dispVec[0] ) / Ld[0])
    L1_v =  float(( ey[1] + dispVec[3] - ey[0] - dispVec[1] ) / Ld[0])
    
    L2_u =  float(( ex[2] + dispVec[4] - ex[1] - dispVec[2] ) / Ld[1])
    L2_v =  float(( ey[2] + dispVec[5] - ey[1] - dispVec[3] ) / Ld[1])
    
    L3_u = float(( ex[0] + dispVec[0] - ex[2] - dispVec[4] ) / Ld[2])
    L3_v = float(( ey[0] + dispVec[1] - ey[2] - dispVec[5] ) / Ld[2])


    B_lv   = np.array([[   -L1_u, -L1_v,  L1_u,  L1_v,     0,      0 ], 
                       [       0,     0, -L2_u, -L2_v,  L2_u,   L2_v ],
                       [    L3_u,  L3_v,     0,     0, -L3_u,  -L3_v ] ])
    
    return B_lv

def get_B_el_matrix(ex,ey, dispVec, m):
    """ 
    B-matrix consisting of partial derivative of natural coordinate strain 
    as a function of length.
    B_el = partial eps / partial L 
    """
    e_n = bar.get_NaturalCoordinates_strain_FD(ex, ey, dispVec, m)

    B_el   = np.array([ [e_n[0],      0,    0  ],
                        [     0, e_n[1],    0  ],
                        [     0,      0, e_n[2]] ])
    return B_el


def get_material_stiffness_ANS_K3(ex,ey, dispVec, C, m):
    """
    Compute the material stiffness matrix for ANS triangle element
    """
    
    B_lv = get_B_lv_matrix(ex,ey, dispVec)
    B_el = get_B_el_matrix(ex,ey, dispVec, m)
    
    #Transformation between cartesian and natural strain  
    T_cn = bar.get_Triangle_element_transformation_matrix(ex, ey, dispVec)
    T_nc = np.linalg.inv(T_cn)
    C_n = T_nc.T @ C @ T_nc

    Km3 = B_lv.T @ B_el.T @ C_n @ B_el @ B_lv
    
    return Km3

def get_B_eLL_s_matrix(ex,ey, C, dispVec, m):
    eps_n = bar.get_NaturalCoordinates_strain(ex, ey, dispVec, m)
    
    #Transformation between cartesian and natural strain  
    T_cn = bar.get_Triangle_element_transformation_matrix(ex, ey, dispVec)
    T_nc = np.linalg.inv(T_cn)
    C_n = T_nc.T @ C @ T_nc

    s_n = C_n @ eps_n
    
    #second derivatives of natural green strain
    e_n = bar.get_NaturalCoordinates_strain_SD(ex, ey, dispVec, m)
    
    B_ells =np.array([ [e_n[0]*s_n[0],             0,             0 ],
                       [            0, e_n[1]*s_n[1],             0 ],
                       [            0,             0, e_n[2]*s_n[2] ] ]) 
    
    return B_ells


def get_geometric_stiffness_ANS_K2(ex,ey, dispVec, C, m):
    
    B_lv = get_B_lv_matrix(ex,ey, dispVec)
    B_eLLs = get_B_eLL_s_matrix(ex,ey, C, dispVec, m)
    
    Kg2 =  B_lv.T @ B_eLLs @ B_lv

    return Kg2



def get_B_r_matrix(ex,ey,dispVec):
    
    Ld = bar.get_length_deformed(ex, ey, dispVec)
    
    ex1 = float(( ex[1] + dispVec[2] - ex[0] - dispVec[0] ) / Ld[0])
    ey1 = float(( ey[1] + dispVec[3] - ey[0] - dispVec[1] ) / Ld[0])
    
    ex2 = float(( ex[2] + dispVec[4] - ex[1] - dispVec[2] ) / Ld[1])
    ey2 = float(( ey[2] + dispVec[5] - ey[1] - dispVec[3] ) / Ld[1])
    
    ex3 = float(( ex[0] + dispVec[0] - ex[2] - dispVec[4] ) / Ld[2])
    ey3 = float(( ey[0] + dispVec[1] - ey[2] - dispVec[5] ) / Ld[2])
    
    
    B_R   = np.array([ [    ey1, -ex1,  -ey1,  ex1,     0,     0 ], 
                       [      0,    0,   ey2, -ex2,  -ey2,   ex2 ],
                       [   -ey3,  ex3,     0,    0,   ey3,  -ex3 ] ])
    
    return B_R

def get_geometric_stiffness_ANS_K1(ex,ey, dispVec, C, m):
    """Compute the Geometric stiffness matrix for ANS-element, green strain, K1"""
    

    Ld   = bar.get_length_deformed(ex, ey, dispVec)
    B_el = get_B_el_matrix(ex,ey, dispVec, m)

    eps_n = bar.get_NaturalCoordinates_strain(ex, ey, dispVec, m)
    
    #Transformation between cartesian and natural strain  
    T_cn = bar.get_Triangle_element_transformation_matrix(ex, ey, dispVec)
    T_nc = np.linalg.inv(T_cn)
    C_n = T_nc.T @ C @ T_nc
    
    s_n = C_n @ eps_n

    SHN = B_el.T @ s_n
    
    S_hn = np.array  ([ [SHN[0]/Ld[0],            0,            0  ],
                        [           0, SHN[1]/Ld[1],            0  ],
                        [           0,            0, SHN[2]/Ld[2]  ] ])
    
    
    B_R = get_B_r_matrix(ex,ey,dispVec)
         
    Kg1 = B_R.T @ S_hn @ B_R
    
    return Kg1


def get_tangent_stiffness_ANS(ex,ey, dispVec, C, m):
    """Compute the tangetn stiffness for green strain ANS element"""
    Kg1 = get_geometric_stiffness_ANS_K1(ex,ey, dispVec, C, m)
    Kg2 = get_geometric_stiffness_ANS_K2(ex,ey, dispVec, C, m)
    Km3 = get_material_stiffness_ANS_K3(ex,ey, dispVec, C, m)

    
    Kt = Kg1 + Kg2 + Km3

    return Kt