# -*- coding: utf-8 -*-

import numpy as np
import math
import HM.source.tri3_tangent_stiffness as tan3
import HM.source.tri3_large_strains as tri3

def get_length_initial(ex, ey, dispVec):
    """Compute the intitial length of the ANS bar element"""
    L0 = np.zeros(3)

    L0a = math.sqrt((ex[1]-ex[0])**2 + (ey[1]-ey[0])**2)
    L0b = math.sqrt((ex[2]-ex[1])**2 + (ey[2]-ey[1])**2)
    L0c = math.sqrt((ex[0]-ex[2])**2 + (ey[0]-ey[2])**2)
    
    L0[0] = L0a
    L0[1] = L0b
    L0[2] = L0c
    
    return L0

def get_length_deformed(ex, ey, dispVec):
    """Compute the deformed length of the ANS bar element"""
    Ld = np.zeros(3)

    La = math.sqrt( ((ex[1]-ex[0]) + (dispVec[2]-dispVec[0]))**2 + ((ey[1]-ey[0])  + (dispVec[3]-dispVec[1]))**2)
    Lb = math.sqrt( ((ex[2]-ex[1]) + (dispVec[4]-dispVec[2]))**2 + ((ey[2]-ey[1])  + (dispVec[5]-dispVec[3]))**2)
    Lc = math.sqrt( ((ex[0]-ex[2]) + (dispVec[0]-dispVec[4]))**2 + ((ey[0]-ey[2])  + (dispVec[1]-dispVec[5]))**2)

    Ld[0] = La
    Ld[1] = Lb
    Ld[2] = Lc
    
    return Ld


def get_Triangle_element_transformation_matrix(ex, ey, dispVec):
    """
    Compute the Transformation matrix for triangle element, consisting of normal strain
    Thogether with unit vector along the bar, the strain of the element could be obtained
    """
    L0 = get_length_initial(ex, ey, dispVec)

    Ca = (ex[1] - ex[0])/L0[0]
    Sa = (ey[1] - ey[0])/L0[0]
    Cb = (ex[2] - ex[1])/L0[1]
    Sb = (ey[2] - ey[1])/L0[1]
    Cc = (ex[0] - ex[2])/L0[2]
    Sc = (ey[0] - ey[2])/L0[2]
    
    T = np.array([     [ Ca**2, Sa**2, Ca*Sa],
                       [ Cb**2, Sb**2, Cb*Sb],
                       [ Cc**2, Sc**2, Cc*Sc]  ]) 
    return T


def get_Transformation_matrix_GL(ex, ey, dispVec, b):
    """Transformation matrix from global to local displacements"""
    L0 = get_length_initial(ex, ey, dispVec)
    
    for i in range(3):

        if b == 1: #BAR 1
            C = (ex[1] - ex[0])/L0[0]    #cos(Theta_a)
            S = (ey[1] - ey[0])/L0[0]    #sin(Theta_a)

        if b ==2: #BAR 2
            C = (ex[2] - ex[1])/L0[1]    #cos(Theta_b)
            S = (ey[2] - ey[1])/L0[1]    #sin(Theta_b)
            
        if b ==3: #BAR 3
            C = (ex[0] - ex[2])/L0[2]   #cos(Theta_c)
            S = (ey[0] - ey[2])/L0[2]   #sin(Theta_c)
            
    T = np.array([ [ C, S ],
                   [-S, C ] ]) 
    return T



def get_Triangle_element_lambda_vector(ex, ey, dispVec):
    """Compute the lambda vector for ANS element"""
    Lambda = np.zeros(3)
    
    L0 = get_length_initial(ex, ey, dispVec)
    L  =get_length_deformed(ex, ey, dispVec)
    for i in range(3):
        Lambda[i] = L[i] / L0[i]
        i += 1  
    
    return Lambda


#--------------------------- Seth-hill strain for ANS element-------------------------

def get_NaturalCoordinates_strain(ex, ey, dispVec, m):
    """Calculate the Seth-hill natural coordinate strains for ANS element"""
    e_n = np.zeros(3)
    
    for i in range(3):
        Lambda = get_Triangle_element_lambda_vector(ex, ey, dispVec)
        
        if m == 2: #Green
            
            e_n[i] = 1/2 * (Lambda[i]**2 - 1)
            i += 1 
            
        elif m == 1: #Biot
            
            e_n[i] = Lambda[i] - 1
            i += 1 

        elif m == 0: #Hencky
            
            e_n[i] = math.log(Lambda[i])
            i += 1 

        elif m == -1: #Swainger
            
            e_n[i] = 1 - 1/Lambda[i]
            i += 1 

        elif m == -2: #Almansi
            
            e_n[i] = (1 - (1/Lambda[i]**2))*1/2
            i += 1 
            
    return e_n


def get_CartesianCoordinates_strain(ex, ey, dispVec, m):
    """Calculate the Seth-hill cartesian coordinate strains for ANS element"""
    e_c = np.zeros(3)
    
    for i in range(3):
        e_n = get_NaturalCoordinates_strain(ex, ey, dispVec, m)
        T_nc = get_Triangle_element_transformation_matrix(ex, ey, dispVec)
        T_cn = np.linalg.inv(T_nc)
        
        e_c =  T_cn @ e_n
    return e_c

#--------------------first derivative strain for ANS element---------------------------

def get_NaturalCoordinates_strain_FD(ex, ey, dispVec, m):
    """Calculate the first derivative of natural coordinate strains for ANS element"""  
    e_n = np.zeros(3)
    
    for i in range(3):
        L0 = get_length_initial(ex, ey, dispVec) 
        L = get_length_deformed(ex, ey, dispVec)
        
        if m == 2: #Green
            
            e_n[i] = L[i] / L0[i]**2
            i += 1 
            
        elif m == 1: #Biot
            
            e_n[i] = 1 / L0[i]
            i += 1 

        elif m == 0: #Hencky
            
            e_n[i] = 1 / L[i]
            i += 1 

        elif m == -1: #Swainger
            
            e_n[i] = L0[i] / L[i]**2
            i += 1 

        elif m == -2: #Almansi
            
            e_n[i] = L0[i]**2 / L[i]**3
            i += 1 
            
    return e_n

def get_CartesianCoordinates_strain_FD(ex, ey, dispVec, m):
    """Calculate the first derivative of cartesian coordinate strains for ANS element"""    
    e_c = np.zeros(3)
    
    for i in range(3):
        e_n =  get_NaturalCoordinates_strain_FD(ex, ey, dispVec, m)
        T_nc = get_Triangle_element_transformation_matrix(ex, ey, dispVec)
        T_cn = np.linalg.inv(T_nc)
        
        e_c =  T_cn @ e_n
        
    return e_c

#------------------------------second derivative strain-------------------------------

def get_NaturalCoordinates_strain_SD(ex, ey, dispVec, m):
    """Calculate the second derivative of natural coordinate strains for ANS element"""        
    e_n = np.zeros(3)
    
    for i in range(3):
        L0 = get_length_initial(ex, ey, dispVec) 
        L = get_length_deformed(ex, ey, dispVec)
        
        if m == 2: #Green
            
            e_n[i] = 1 / L0[i]**2
            i += 1 
            
        elif m == 1: #Biot
            
            e_n[i] = 0
            i += 1 

        elif m == 0: #Hencky
            
            e_n[i] = - 1 / L[i]**2
            i += 1 

        elif m == -1: #Swainger
            
            e_n[i] = -2 * L0[i] / L[i]**3
            i += 1 

        elif m == -2: #Almansi
            
            e_n[i] = -3 * L0[i]**2 / L[i]**4
            i += 1 
            
    return e_n

def get_CartesianCoordinates_strain_SD(ex, ey, dispVec, m):
    """Calculate the second derivative of cartesian coordinate strains for ANS element"""    
    
    e_c = np.zeros(3)
    
    for i in range(3):
        e_n =  get_NaturalCoordinates_strain_SD(ex, ey, dispVec, m)
        T_nc = get_Triangle_element_transformation_matrix(ex, ey, dispVec)
        T_cn = np.linalg.inv(T_nc)
        
        e_c =  T_cn @ e_n
    return e_c

#--------------------------------2D motion for bar-element------------------------------

def get_local_displacement_2D(ex, ey, dispVec):
    """Go from global to local displacement for 2D-motion ANS element"""
    
    L0 = get_length_initial(ex, ey, dispVec)

    #Local disp for first bar
    T_bar1 = get_Transformation_matrix_GL(ex, ey, dispVec, 1)
        
    dispLoc1_bar1 = T_bar1 @ dispVec[:2] #u1, v1
    dispLoc2_bar1 = T_bar1 @ dispVec[2:4] #u2, v2
    
    g1_loc_1bar = ( dispLoc2_bar1[0] - dispLoc1_bar1[0]) /L0[0]
    g2_loc_1bar = ( dispLoc2_bar1[1] - dispLoc1_bar1[1]) /L0[0] 
        
    #Local disp for second bar
    T_bar2 = get_Transformation_matrix_GL(ex, ey, dispVec, 2)
    
    dispLoc1_bar2 = T_bar2 @ dispVec[2:4] #u2, v2
    dispLoc2_bar2 = T_bar2 @ dispVec[4:] #u3, v3
        
    g1_loc_2bar = ( dispLoc2_bar2[0] - dispLoc1_bar2[0]) /L0[1]
    g2_loc_2bar = ( dispLoc2_bar2[1] - dispLoc1_bar2[1]) /L0[1] 
        
    ##Local disp for third bar
    T_bar3 = get_Transformation_matrix_GL(ex, ey, dispVec, 3)
    
    dispLoc1_bar3 = T_bar3 @ dispVec[4:] #u3, v3
    dispLoc2_bar3 = T_bar3 @ dispVec[:2] #u1, v1
        
    g1_loc_3bar = ( dispLoc2_bar3[0] - dispLoc1_bar3[0]) /L0[2]
    g2_loc_3bar = ( dispLoc2_bar3[1] - dispLoc1_bar3[1]) /L0[2] 
    
    #displacement gradients on vector form
    g1_local = np.array([g1_loc_1bar, g1_loc_2bar, g1_loc_3bar])
    g2_local = np.array([g2_loc_1bar, g2_loc_2bar, g2_loc_3bar])
    
    return g1_local, g2_local

def get_NaturalCoordinates_strain_2D(ex, ey, dispVec, m):
    """Calculate cartesian strain for ANS 2D-motion"""
    
    e_n = np.zeros(3)
    
    for i in range(3):

        g1_loc, g2_loc =get_local_displacement_2D(ex, ey, dispVec)

        if m == 2:
            
            e_n[0] = g1_loc[0] + (g1_loc[0]**2 + g2_loc[0]**2) /2.0
            e_n[1] = g1_loc[1] + (g1_loc[1]**2 + g2_loc[1]**2) /2.0
            e_n[2] = g1_loc[2] + (g1_loc[2]**2 + g2_loc[2]**2) /2.0
 
    return e_n


def get_CartesianCoordinates_strain_2D(ex, ey, dispVec, m):
    """Calculate cartesian strain for ANS 2D-motion""" 
    e_c = np.zeros(3)
    
    for i in range(3):
        e_n =  get_NaturalCoordinates_strain_2D(ex, ey, dispVec, m)
        T_nc = get_Triangle_element_transformation_matrix(ex, ey, dispVec)
        T_cn = np.linalg.inv(T_nc)
        
        e_c =  T_cn @ e_n
    return e_c



def get_NaturalCoordinates_strain_2D_FD(ex, ey, dispVec, m):
    """Calculate natural strain for first derivative of ANS 2D-motion"""
    e_n = np.zeros(3)
    
    for i in range(3):

        g1_loc, g2_loc =get_local_displacement_2D(ex, ey, dispVec)

        if m == 2:
            
            e_n[0] = (g1_loc[0] +  1) +  g2_loc[0]
            e_n[1] = (g1_loc[1] +  1) +  g2_loc[1]
            e_n[2] = (g1_loc[2] +  1) +  g2_loc[2]

    return e_n


def get_CartesianCoordinates_strain_2D_FD(ex, ey, dispVec, m):
    """Calculate cartesian strain for first derivative of ANS 2D-motion"""
    e_c = np.zeros(3)
    
    for i in range(3):
        e_n =  get_NaturalCoordinates_strain_2D_FD(ex, ey, dispVec, m)
        T_nc = get_Triangle_element_transformation_matrix(ex, ey, dispVec)
        T_cn = np.linalg.inv(T_nc)
        
        e_c =  T_cn @ e_n
    return e_c


#BMATRIX FOR ANS ELEMENT::

def get_Bmat_eps_gVec(ex, ey, dispVec, n):
    """
    Compute B_eg for ANS element
    Index n represents the bar number
    """
    if n == 1:   #bar1
        g1_loc, g2_loc = get_local_displacement_2D(ex, ey, dispVec)
        Bmat = np.array([ [(1.0+float(g1_loc[0])), float(g2_loc[0]) ] ])
    
    if n ==2:   #bar2
        g1_loc, g2_loc = get_local_displacement_2D(ex, ey, dispVec)
        Bmat = np.array([ [(1.0+float(g1_loc[1])), float(g2_loc[1]) ] ])
        
    if n==3:    #bar3
        g1_loc, g2_loc = get_local_displacement_2D(ex, ey, dispVec)
        Bmat = np.array([ [(1.0+float(g1_loc[2])), float(g2_loc[2]) ] ])
        
    return Bmat

def get_Bmat_gVecL_dispL(ex,ey,dispVec):
    """B-matrix g in local from disp in local """
    f = np.zeros(3)
    L0 = get_length_initial(ex, ey, dispVec)
    
    f[0] = 1/L0[0]
    f[1] = 1/L0[1]
    f[2] = 1/L0[2]
    Bmat_bar1 = np.array([[-f[0],0,f[0],0],[0,-f[0],0,f[0]]])
    Bmat_bar2 = np.array([[-f[1],0,f[1],0],[0,-f[1],0,f[1]]])
    Bmat_bar3 = np.array([[-f[2],0,f[2],0],[0,-f[2],0,f[2]]])
    
    return Bmat_bar1, Bmat_bar2, Bmat_bar3


def get_Bmat_gVecL_dispG(ex, ey, dispVec):
    """B-matrix g in local from disp in global """
    
    BmatL_bar1, BmatL_bar2, BmatL_bar3 = get_Bmat_gVecL_dispL(ex,ey,dispVec)
    
    T1 = get_Transformation_matrix_GL(ex, ey, dispVec, 1)
    T2 = get_Transformation_matrix_GL(ex, ey, dispVec, 2)
    T3 = get_Transformation_matrix_GL(ex, ey, dispVec, 3)
    
    
    #bar1
    T_bar1 = np.zeros((4,4))
    T_bar1[0:2,0:2] = T1
    T_bar1[2:4,2:4] = T1
    BmatG_bar1 = BmatL_bar1 @ T_bar1
    
    #bar2
    T_bar2 = np.zeros((4,4))
    T_bar2[0:2,0:2] = T2
    T_bar2[2:4,2:4] = T2
    BmatG_bar2 = BmatL_bar2 @ T_bar2
    
    #bar3
    T_bar3 = np.zeros((4,4))
    T_bar3[0:2,0:2] = T3
    T_bar3[2:4,2:4] = T3
    BmatG_bar3 = BmatL_bar3 @ T_bar3

    return BmatG_bar1, BmatG_bar2, BmatG_bar3

def get_Bmat_nv_ANS(ex, ey, dispVec):
    """Calculate the natural strain-displacement matrix from ANS formulation"""
    B_eps_bar1 = get_Bmat_eps_gVec(ex, ey, dispVec, 1)
    B_eps_bar2 = get_Bmat_eps_gVec(ex, ey, dispVec, 2)
    B_eps_bar3 = get_Bmat_eps_gVec(ex, ey, dispVec, 3)
    BmatG_bar1, BmatG_bar2, BmatG_bar3 = get_Bmat_gVecL_dispG(ex, ey, dispVec)
    
    B1_1 = B_eps_bar1
    B1_2 = BmatG_bar1
    B1 = B1_1 @ B1_2
    
    B2_1 = B_eps_bar2
    B2_2 = BmatG_bar2
    B2 = B2_1 @ B2_2
    
    B3_1 = B_eps_bar3
    B3_2 = BmatG_bar3
    B3 = B3_1 @ B3_2

    B_nv = np.array([[B1[0,0], B1[0,1], B1[0,2], B1[0,3],     0.0,     0.0],
                     [    0.0,     0.0, B2[0,0], B2[0,1], B2[0,2], B2[0,3]],
                     [B3[0,2], B3[0,3],     0.0,     0.0, B3[0,0], B3[0,1]]])
    return B_nv

def get_Bmat_cv_ANS(ex, ey, dispVec):
        """ Calculate the Cartesian strain-displacement matrix from ANS formulation"""
        Bnv = get_Bmat_nv_ANS(ex, ey, dispVec)
        T_nc = get_Triangle_element_transformation_matrix(ex, ey, dispVec)
        T_cn = np.linalg.inv(T_nc)
        Bcv = T_cn @ Bnv
        return Bcv
    
    
#----------------------------------Bar-element------------------------------------


def get_partial_strain_displacement_tensor_green(ex,ey, dispVec):
    """Partial derivative of cartesian strain with respect to g
    Returns B-matrix of tradisional strain"""

    B_eg = tan3.get_B_eg_matrix_GreenStrain(ex,ey,dispVec)
    Disp_grad = tri3.get_Disp_grad(ex, ey)
    
    B_mat = B_eg @ Disp_grad

    return B_mat



















