import numpy as np
import HM.source.geom_2D as geom


def C_glob(emod, enu):
    '''
    Funtion for establishing constitutive material stiffness matrix C for linear elastic isotropic material

    :param emod: float, E-modulus
    :param enu:  float, Poisson ratio
    :return:     np.array (6,6), constitutive material stiffness matrix C
    '''

    fac = emod / ((1 + enu) * (1 - 2 * enu))

    C = np.array([
        [(1.0 - enu) * fac, enu * fac, 0.0],
        [enu * fac, (1.0 - enu) * fac, 0.0],
        [0.0, 0.0, (0.5 - enu) * fac]
    ])

    return C

def C_nat(ex, ey, emod, enu):
    '''
    Transforms constitutive stiffness matrix
    The relationships between natural strains e_n and global strains e is written as:

    e_n = Te_inv @ e
    e   = Te @ e_n

    Using energy relation:
    delta_E = delta_e.T @ sigma
            = delta_e.T @ C @ e
            = delta_e_n.T @ Te.T @ C @ Te @ e_n

    C_n = Te.T @ C @ Te

    :param ex:          np.array (3,), element x-coordinates [x0, x1, x2]
    :param ey:          np.array (3,), element y-coordinates
    :param emod:        float, element E-modulus
    :param enu:         float, element Poisson
    :return:            np.array (3,3), transformed constitutive material stiffness matrix C_n
    '''

    # Forming the inverse transformation matrix Te_inv
    # Need the side unit vectors sij, which are defined along edge between node i and j

    s01 = geom.sideVector(ex, ey, fromNode=0, toNode=1, unitVector=True)
    s12 = geom.sideVector(ex, ey, fromNode=1, toNode=2, unitVector=True)
    s20 = geom.sideVector(ex, ey, fromNode=2, toNode=1, unitVector=True)

    Te_inv = np.array([
        [s01[0] ** 2, s01[1] ** 2, s01[0] * s01[1]],
        [s12[0] ** 2, s12[1] ** 2, s12[0] * s12[1]],
        [s20[0] ** 2, s20[1] ** 2, s20[0] * s20[1]]
    ])

    C = C_glob(emod, enu)
    Te = np.linalg.inv(Te_inv)
    C_n = Te.T @ C @ Te

    return C_n
