import numpy as np
import HM.source.geom_3D as geom


def C_glob(emod, enu):
    '''
    Funtion for establishing constitutive material stiffness matrix C for linear elastic isotropic material

    :param emod: float, E-modulus
    :param enu:  float, Poisson ratio
    :return:     np.array (6,6), constitutive material stiffness matrix C
    '''

    fac = emod / ((1 + enu) * (1 - 2 * enu))

    C = np.array([
        [(1.0 - enu) * fac, enu * fac, enu * fac, 0.0, 0.0, 0.0],
        [enu * fac, (1.0 - enu) * fac, enu * fac, 0.0, 0.0, 0.0],
        [enu * fac, enu * fac, (1.0 - enu) * fac, 0.0, 0.0, 0.0],
        [0.0, 0.0, 0.0, (0.5 - enu) * fac, 0.0, 0.0],
        [0.0, 0.0, 0.0, 0.0, (0.5 - enu) * fac, 0.0],
        [0.0, 0.0, 0.0, 0.0, 0.0, (0.5 - enu) * fac]
    ])

    return C


def C_nat(ex, ey, ez, emod, enu):
    '''
    Transforms constitutive stiffness matrix
    The relationships between natural strains e_n and global strains e is written as:

    e_n = Te_inv @ e
    e   = Te @ e_n

    Using energy relation:
    delta_E = delta_e.T @ sigma
            = delta_e.T @ C @ e
            = delta_e_n.T @ Te.T @ C @ Te @ e_n

    C_n = Te.T @ C @ Te

    See page 17 in haugen_tetra_main for Te_inv

    :param ex:          np.array (4,), element x-coordinates [x0, x1, x2, x3]
    :param ey:          np.array (4,), element y-coordinates
    :param ez:          np.array (4,), element z-coordinates
    :param emod:        float, element E-modulus
    :param enu:         float, element Poisson
    :return:            np.array (6,6), transformed constitutive material stiffness matrix C_n
    '''

    # Forming the inverse transformation matrix Te_inv
    # Need the side unit vectors sij, which are defined along edge between node i and j

    s01 = geom.sideVector(ex, ey, ez, fromNode=0, toNode=1, unitVector=True)
    s12 = geom.sideVector(ex, ey, ez, fromNode=1, toNode=2, unitVector=True)
    s23 = geom.sideVector(ex, ey, ez, fromNode=2, toNode=3, unitVector=True)
    s30 = geom.sideVector(ex, ey, ez, fromNode=3, toNode=0, unitVector=True)
    s02 = geom.sideVector(ex, ey, ez, fromNode=0, toNode=2, unitVector=True)
    s13 = geom.sideVector(ex, ey, ez, fromNode=1, toNode=3, unitVector=True)

    Te_inv = np.array([
        [s01[0] ** 2, s01[1] ** 2, s01[2] ** 2, s01[0] * s01[1], s01[1] * s01[2], s01[2] * s01[0]],
        [s12[0] ** 2, s12[1] ** 2, s12[2] ** 2, s12[0] * s12[1], s12[1] * s12[2], s12[2] * s12[0]],
        [s23[0] ** 2, s23[1] ** 2, s23[2] ** 2, s23[0] * s23[1], s23[1] * s23[2], s23[2] * s23[0]],
        [s30[0] ** 2, s30[1] ** 2, s30[2] ** 2, s30[0] * s30[1], s30[1] * s30[2], s30[2] * s30[0]],
        [s02[0] ** 2, s02[1] ** 2, s02[2] ** 2, s02[0] * s02[1], s02[1] * s02[2], s02[2] * s02[0]],
        [s13[0] ** 2, s13[1] ** 2, s13[2] ** 2, s13[0] * s13[1], s13[1] * s13[2], s13[2] * s13[0]]
    ])

    C = C_glob(emod, enu)
    Te = np.linalg.inv(Te_inv)
    C_n = Te.T @ C @ Te

    return C_n
