import numpy as np


def sideVector(ex, ey, ez, fromNode, toNode, unitVector=True, giveLength=False):
    '''
    Creates a vector between given nodes. Options to return as unit vector and length

    :param ex:          np.array (4,), element x-coordinates [x0, x1, x2, x3]
    :param ey:          np.array (4,), element y-coordinates
    :param ez:          np.array (4,), element z-coordinates
    :param fromNode:    integer, starting node for vector
    :param toNode:      integer, end node for vector
    :param unitVector:  boolean, if True the vector is normalised
    :param giveLength:  boolean, if True the sideVector() returns tuple (vector, length)
    :return:            np.array (3,), vector between nodes
    '''
    sv = np.asarray((ex[toNode] - ex[fromNode], ey[toNode] - ey[fromNode], ez[toNode] - ez[fromNode]))

    if unitVector:
        length = np.linalg.norm(sv)
        sv = sv / length
        if giveLength:
            return sv, length

    return sv


def partials_and_vol(ex, ey, ez):
    '''
    Calculates the partial differentials wrt. x,y,z and element volume

    :param ex:          np.array (4,), element x-coordinates [x0, x1, x2, x3]
    :param ey:          np.array (4,), element y-coordinates
    :param ez:          np.array (4,), element z-coordinates
    :return:            tuple (np.array, np.array, np.array, float)
    '''
    cartesian = np.array([
        [1, 1, 1, 1],
        [ex[0], ex[1], ex[2], ex[3]],
        [ey[0], ey[1], ey[2], ey[3]],
        [ez[0], ez[1], ez[2], ez[3]]
    ])

    vol = abs(np.linalg.det(cartesian)) / 6
    natural = np.linalg.inv(cartesian)

    a = natural[:, 1]
    b = natural[:, 2]
    c = natural[:, 3]

    return a, b, c, vol
