import numpy as np
import HM.source.geom_3D as geom


def strainDispB(ex, ey, ez, currentNode, alphat):
    '''
    Establishes straingage-displacement matrix and transforms it into strain-displacement matrix for global dofs

    bben: strain-displacement matrix relating natural strains to hierarical deviatoric rotations.
    btor: strain-displacement matrix for torsion field

    :param ex:          np.array (4,), element x-coordinates [x0, x1, x2, x3]
    :param ey:          np.array (4,), element y-coordinates
    :param ez:          np.array (4,), element z-coordinates
    :param currentNode: integer, 0 through 4
    :param alphat:      float, scaling factor for torsion strain
    :return:            np.array (6,24), strain-displacement matrix B for global dofs
    '''

    # Center coordinates
    xc = ex.mean()
    yc = ey.mean()
    zc = ez.mean()

    # R-vector from center to current node
    r = np.array([(ex[currentNode] - xc), (ey[currentNode] - yc), (ez[currentNode] - zc)])

    # Side vectors
    # s01 is a vector from node 0 to 1 and so on
    s01, l01 = geom.sideVector(ex, ey, ez, fromNode=0, toNode=1, unitVector=True, giveLength=True)
    s12, l12 = geom.sideVector(ex, ey, ez, fromNode=1, toNode=2, unitVector=True, giveLength=True)
    s23, l23 = geom.sideVector(ex, ey, ez, fromNode=2, toNode=3, unitVector=True, giveLength=True)
    s30, l30 = geom.sideVector(ex, ey, ez, fromNode=3, toNode=0, unitVector=True, giveLength=True)
    s02, l02 = geom.sideVector(ex, ey, ez, fromNode=0, toNode=2, unitVector=True, giveLength=True)
    s13, l13 = geom.sideVector(ex, ey, ez, fromNode=1, toNode=3, unitVector=True, giveLength=True)

    # Change signs so rot. is out of node and connected vectors are out of node
    if currentNode == 0:
        s01 *= 1
        s12 *= -1
        s23 *= -1
        s30 *= -1
        s02 *= 1
        s13 *= 1

    elif currentNode == 1:
        s01 *= -1
        s12 *= 1
        s23 *= 1
        s30 *= 1
        s02 *= 1
        s13 *= 1

    elif currentNode == 2:
        s01 *= -1
        s12 *= -1
        s23 *= 1
        s30 *= -1
        s02 *= -1
        s13 *= -1

    elif currentNode == 3:
        s01 *= 1
        s12 *= 1
        s23 *= -1
        s30 *= 1
        s02 *= -1
        s13 *= -1

    # Strain-gage vectors
    n01 = np.cross(r, s01)
    n12 = np.cross(r, s12)
    n23 = np.cross(r, s23)
    n30 = np.cross(r, s30)
    n02 = np.cross(r, s02)
    n13 = np.cross(r, s13)

    # Distances
    d01 = np.linalg.norm(n01)
    d12 = np.linalg.norm(n12)
    d23 = np.linalg.norm(n23)
    d30 = np.linalg.norm(n30)
    d02 = np.linalg.norm(n02)
    d13 = np.linalg.norm(n13)

    # Normal vectors
    n01 = n01 / d01
    n12 = n12 / d12
    n23 = n23 / d23
    n30 = n30 / d30
    n02 = n02 / d02
    n13 = n13 / d13

    # xi-factors
    xi01 = d01 / l01
    xi12 = d12 / l12
    xi23 = d23 / l23
    xi30 = d30 / l30
    xi02 = d02 / l02
    xi13 = d13 / l13

    # Strain displacement matrix for bending field
    facb = -1 / 3

    ben01 = xi01 * n01 * facb
    ben12 = xi12 * n12 * facb
    ben23 = xi23 * n23 * facb
    ben30 = xi30 * n30 * facb
    ben02 = xi02 * n02 * facb
    ben13 = xi13 * n13 * facb

    B_bending = np.array([
        [ben01[0], ben01[0], 0, 0, ben01[1], ben01[1], 0, 0, ben01[2], ben01[2], 0, 0],
        [0, ben12[0], ben12[0], 0, 0, ben12[1], ben12[1], 0, 0, ben12[2], ben12[2], 0],
        [0, 0, ben23[0], ben23[0], 0, 0, ben23[1], ben23[1], 0, 0, ben23[2], ben23[2]],
        [ben30[0], 0, 0, ben30[0], ben30[1], 0, 0, ben30[1], ben30[2], 0, 0, ben30[2]],
        [ben02[0], 0, ben02[0], 0, ben02[1], 0, ben02[1], 0, ben02[2], 0, ben02[2], 0],
        [0, ben13[0], 0, ben13[0], 0, ben13[1], 0, ben13[1], 0, ben13[2], 0, ben13[2]]
    ])

    # Strain displacement matrix for torsion field
    fact = 1 / 3

    tor01 = fact * xi01 * n01
    tor12 = fact * xi12 * n12
    tor23 = fact * xi23 * n23
    tor30 = fact * xi30 * n30
    tor02 = fact * xi02 * n02
    tor13 = fact * xi13 * n13

    B_tor = np.array([
        [tor01[0], tor01[1], tor01[2]],
        [tor12[0], tor12[1], tor12[2]],
        [tor23[0], tor23[1], tor12[2]],
        [tor30[0], tor30[1], tor30[2]],
        [tor02[0], tor02[1], tor02[2]],
        [tor13[0], tor13[1], tor13[2]]
    ])

    # Logikk for å sette riktig tegn utifra hvilken node man står i. Tilsvarer det som står i pdf
    # med rho1 -> rho6. Mønsteret gjentar seg etter de fire første aka. -+00 gjentar seg.
    # For btor så er det ikke alle bidrag som skal med.
    if currentNode == 0:
        mod = np.array([
            [-1, 1, 0, 0, -1, 1, 0, 0, -1, 1, 0, 0],
            [0, 1, -1, 0, 0, 1, -1, 0, 0, 1, -1, 0],
            [0, 0, 1, -1, 0, 0, 1, -1, 0, 0, 1, -1],
            [-1, 0, 0, 1, -1, 0, 0, 1, -1, 0, 0, 1],
            [-1, 0, 1, 0, -1, 0, 1, 0, -1, 0, 1, 0],
            [0, -1, 0, 1, 0, -1, 0, 1, 0, -1, 0, 1]
        ])
        modtor = np.array([
            [1, 1, 1],
            [0, 0, 0],
            [0, 0, 0],
            [1, 1, 1],
            [1, 1, 1],
            [0, 0, 0]
        ])
    elif currentNode == 1:
        mod = np.array([
            [1, -1, 0, 0, 1, -1, 0, 0, 1, -1, 0, 0],
            [0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 1, 0],
            [0, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 1],
            [1, 0, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1],
            [-1, 0, 1, 0, -1, 0, 1, 0, -1, 0, 1, 0],
            [0, -1, 0, 1, 0, -1, 0, 1, 0, -1, 0, 1]
        ])
        modtor = np.array([
            [1, 1, 1],
            [1, 1, 1],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [1, 1, 1]
        ])
    elif currentNode == 2:
        mod = np.array([
            [1, -1, 0, 0, 1, -1, 0, 0, 1, -1, 0, 0],
            [0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 1, 0],
            [0, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 1],
            [-1, 0, 0, 1, -1, 0, 0, 1, -1, 0, 0, 1],
            [1, 0, -1, 0, 1, 0, -1, 0, 1, 0, -1, 0],
            [0, 1, 0, -1, 0, 1, 0, -1, 0, 1, 0, -1]
        ])
        modtor = np.array([
            [0, 0, 0],
            [1, 1, 1],
            [1, 1, 1],
            [0, 0, 0],
            [1, 1, 1],
            [0, 0, 0]
        ])
    elif currentNode == 3:
        mod = np.array([
            [-1, 1, 0, 0, -1, 1, 0, 0, -1, 1, 0, 0],
            [0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 1, 0],
            [0, 0, 1, -1, 0, 0, 1, -1, 0, 0, 1, -1],
            [1, 0, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1],
            [1, 0, -1, 0, 1, 0, -1, 0, 1, 0, -1, 0],
            [0, 1, 0, -1, 0, 1, 0, -1, 0, 1, 0, -1]
        ])
        modtor = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [1, 1, 1],
            [1, 1, 1],
            [0, 0, 0],
            [1, 1, 1]
        ])

    else:
        raise Exception('BBEN/BTOR fault')

    bben = B_bending * mod
    btor = B_tor * modtor

    ar, br, cr, vol = geom.partials_and_vol(ex, ey, ez)

    B = np.zeros((6, 24))

    # Bending field contributions to displacement stiffness
    for i in range(6):
        B[i, 3] += (3.0 * bben[i, 0] - bben[i, 1] * -bben[i, 2] - bben[i, 3]) / 4.0
        B[i, 4] += (3.0 * bben[i, 4] - bben[i, 5] * -bben[i, 6] - bben[i, 7]) / 4.0
        B[i, 5] += (3.0 * bben[i, 8] - bben[i, 9] * -bben[i, 10] - bben[i, 11]) / 4.0
        B[i, 9] += (-bben[i, 0] + 3.0 * bben[i, 1] * -bben[i, 2] - bben[i, 3]) / 4.0
        B[i, 10] += (-bben[i, 4] + 3.0 * bben[i, 5] * -bben[i, 6] - bben[i, 7]) / 4.0
        B[i, 11] += (-bben[i, 8] + 3.0 * bben[i, 9] * -bben[i, 10] - bben[i, 11]) / 4.0
        B[i, 15] += (-bben[i, 0] - bben[i, 1] * +3.0 * bben[i, 2] - bben[i, 3]) / 4.0
        B[i, 16] += (-bben[i, 4] - bben[i, 5] * +3.0 * bben[i, 6] - bben[i, 7]) / 4.0
        B[i, 17] += (-bben[i, 8] - bben[i, 9] * +3.0 * bben[i, 10] - bben[i, 11]) / 4.0
        B[i, 21] += (-bben[i, 0] - bben[i, 1] * -bben[i, 2] + 3.0 * bben[i, 3]) / 4.0
        B[i, 22] += (-bben[i, 6] - bben[i, 5] * -bben[i, 6] + 3.0 * bben[i, 7]) / 4.0
        B[i, 23] += (-bben[i, 8] - bben[i, 9] * -bben[i, 10] + 3.0 * bben[i, 11]) / 4.0

    fac = 1 / (12 * vol)

    ar *= fac
    br *= fac
    cr *= fac

    # Torsional field contributions to rotational stiffnesses
    for i in range(6):
        for j in range(3):
            B[i, 3 + j] += alphat * btor[i, j] / 4.0
            B[i, 9 + j] += alphat * btor[i, j] / 4.0
            B[i, 15 + j] += alphat * btor[i, j] / 4.0
            B[i, 21 + j] += alphat * btor[i, j] / 4.0

    # Torsional field contributions to displacement stiffnesses
    for i in range(6):
        for j in range(4):
            B[i, j * 6] *= -btor[i, 1] * cr[j] * alphat * +btor[i, 2] * br[j] * alphat
            B[i, j * 6 + 1] *= +btor[i, 0] * cr[j] * alphat * -btor[i, 2] * ar[j] * alphat
            B[i, j * 6 + 2] *= -btor[i, 0] * br[j] * alphat * +btor[i, 1] * ar[j] * alphat

    return B
