# -*- coding: utf-8 -*-

import numpy as np
import math


def get_B_matrix_linear(ex,ey):
    """
    Compute the element strain displacement matrix for a triangle element
    :return mat Be: element strain displacement matrix [3 x 6]
    """
    
    tmp = np.array([[1,ex[0],ey[0]],
                     [1,ex[1],ey[1]],
                     [1,ex[2],ey[2]]])
    
    A2 = np.linalg.det(tmp)  # Double of triangle area
       
    cyclic_ijk = [0,1,2,0,1]      # Cyclic permutation of the nodes i,j,k
    
    zi_px = np.zeros(3)           # Partial derivative with respect to x
    zi_py = np.zeros(3)           # Partial derivative with respect to y
    
    for i in range(3):
        j = cyclic_ijk[i+1]
        k = cyclic_ijk[i+2]
        zi_px[i] = (ey[j] - ey[k]) / A2
        zi_py[i] = (ex[k] - ex[j]) / A2
        
    Be = np.array([
            [zi_px[0],        0, zi_px[1],        0, zi_px[2],       0],
            [       0, zi_py[0],        0, zi_py[1],        0,zi_py[2]],
            [zi_py[0], zi_px[0], zi_py[1], zi_px[1], zi_py[2], zi_px[2]]])

    return Be


def get_Disp_grad(ex,ey):
    """
    Compute the Displacement gradient matrix for a triangle element
    :return mat Disp_grad: element strain displacement matrix [4 x 6]
    """
    
    tmp = np.array([[1,ex[0],ey[0]],
                     [1,ex[1],ey[1]],
                     [1,ex[2],ey[2]]])
    
    A2 = np.linalg.det(tmp)  # Double of triangle area
       
    cyclic_ijk = [0,1,2,0,1]      # Cyclic permutation of the nodes i,j,k
    
    zi_px = np.zeros(3)           # Partial derivative with respect to x
    zi_py = np.zeros(3)           # Partial derivative with respect to y
    
    for i in range(3):
        j = cyclic_ijk[i+1]
        k = cyclic_ijk[i+2]
        zi_px[i] = (ey[j] - ey[k]) / A2
        zi_py[i] = (ex[k] - ex[j]) / A2
    
    Bg = np.array([
        [zi_px[0],         0, zi_px[1],        0, zi_px[2],        0],
        [       0,  zi_px[0],        0, zi_px[1],        0, zi_px[2]],
        [zi_py[0],         0, zi_py[1],        0, zi_py[2],        0],
        [      0,   zi_py[0],        0, zi_py[1],        0, zi_py[2]]])
    

    return Bg


def get_Disp_grad_Vec(ex,ey,dispVec):
    """
    Compute the displacement gradient vector for a triangle element.
    :return g: element strains [epsXX,epsYY,gammaXY]
    """
    Dispmat = get_Disp_grad(ex,ey)
    gVec = Dispmat @ dispVec
    
    return gVec


#Seth-hill Family strains


def get_strainVec_trad(ex,ey,dispVec, m):
    """Compute the strain vector for Seth-hill family strain"""
    
    for i in range(3):
        gVec = get_Disp_grad_Vec(ex,ey,dispVec)
        eVec = np.zeros(3)
        Bmat = get_B_matrix_linear(ex,ey)
        
        if m ==2: #Green 
            eVec[0] = gVec[0]+ (gVec[0]**2 + gVec[1]**2)/2.0
            eVec[1] = gVec[3]+ (gVec[3]**2 + gVec[2]**2)/2.0
            eVec[2] = gVec[1]+ gVec[2] + gVec[0]*gVec[2] + gVec[1]*gVec[3]
            
        elif m == 1: #Biot
            eVec = Bmat @ dispVec
            
        elif m == 0: #Hencky
            eVec[0] = math.log(1 + gVec[0])
            eVec[1] = math.log(1 + gVec[3])
            eVec[2] = math.log(1 + gVec[1] + gVec[2])

        elif m == -1: #Swainger
            eVec[0] = gVec[0] / (1 + gVec[0])
            eVec[1] = gVec[3] / (1 + gVec[3])
            eVec[2] =  (gVec[1]+gVec[2]) / (1 + gVec[1]+gVec[2])
            
            
        elif m == -2: #Almansi
            eVec[0] = ((1 + gVec[0])**-2 - 1) * -1/2
            eVec[1] = ((1 + gVec[3])**-2 - 1) * -1/2
            eVec[2] = ((1 + gVec[1]+gVec[2])**-2 - 1) * -1/2
    
    return eVec


#First derivatives:
    
def get_strainVec_trad_FD(ex,ey,dispVec, m):
    """Compute the strain vector for the first derivative of Seth-hill family strain"""
    
    for i in range(3):
        gVec = get_Disp_grad_Vec(ex,ey,dispVec)
        eVec = np.zeros(3)
        
        if m ==2: #Green 
            eVec[0] = (gVec[0]+ 1) + gVec[1]
            eVec[1] = (gVec[3]+ 1) + gVec[2]
            eVec[2] =  gVec[2] + (1 + gVec[0]) + (1 + gVec[3]) + gVec[1]
            
        elif m == 1: #Biot
            eVec[0] = 1
            eVec[1] = 1
            eVec[2] = 1
            
        elif m == 0: #Hencky
            eVec[0] = 1 / (1 + gVec[0])
            eVec[1] = 1 / (1 + gVec[3])
            eVec[2] =  1 / (1 + gVec[1] + gVec[2])
            
        elif m == -1: #Swainger
            eVec[0] = 1 / (1 + gVec[0])**2
            eVec[1] = 1 / (1 + gVec[3])**2
            eVec[2] = 1 / (1 + gVec[1] + gVec[2])**2
            
            
        elif m == -2: #Almansi
            eVec[0] = ( -2 * (1 + gVec[0])**-3 ) * - 1/2
            eVec[1] = ( -2 * (1 + gVec[3])**-3 ) * - 1/2 
            eVec[2] = ( -2 * (1 + gVec[1] + gVec[2])**-3 ) * - 1/2
    return eVec


# Second derivatives
    
def get_strainVec_trad_SD(ex,ey,dispVec, m):
    """Compute the strain vector for the second derivative of Seth-hill family strain"""
    
    for i in range(3):
        gVec = get_Disp_grad_Vec(ex,ey,dispVec)
        eVec = np.zeros(3)
        
        if m ==2: #Green 
            eVec[0] = 2
            eVec[1] = 2
            eVec[2] = 4
            
        elif m == 1: #Biot
            eVec[0] = 0
            eVec[1] = 0
            eVec[2] = 0
            
        elif m == 0: #Hencky
            eVec[0] = - 1 / (1 + gVec[0])**2
            eVec[1] = - 1 / (1 + gVec[3])**2
            #eVec[2] = 4
            eVec[2] =- 1 / (1 + gVec[1] + gVec[2])**2
            
        elif m == -1: #Swainger
            eVec[0] = - 2 / (1 + gVec[0])**3
            eVec[1] = - 2 / (1 + gVec[3])**3
            eVec[2] = - 2 / (1 + gVec[1] + gVec[2])**3
            
            
        elif m == -2: #Almansi
            eVec[0] = ( 6 * (1 + gVec[0])**-4 ) * - 1/2
            eVec[1] = ( 6 * (1 + gVec[3])**-4 ) * - 1/2
            eVec[2] = ( 6 * (1 + gVec[1] + gVec[2])**-4 ) * - 1/2
    
    return eVec