import numpy as np
import HM.source.tri3_large_strains as tri3


# -----------------Internal-force-------------------------------

def get_internal_force_trad(ex, ey, dispVec, C, m):
    """Compute the internal forces for green strain"""

    B_gv = tri3.get_Disp_grad(ex, ey)
    B_eg = get_B_eg_matrix_trad(ex, ey, dispVec, m)

    eps = tri3.get_strainVec_trad(ex, ey, dispVec, m)
    s = C @ eps

    fe = B_gv.T @ B_eg.T @ s
    return fe


# --------------------------Tangent stiffness------------------------


def get_B_eg_matrix_trad(ex, ey, dispVec, m):
    """
    Compute the B_eg matrix for traditional strain.
    Consist of partial derivative of strain with respect to g
    """
    g = np.zeros(4)
    g = tri3.get_Disp_grad_Vec(ex, ey, dispVec)

    if m == 2:
        Beg_mat = np.array([[1 + g[0], g[1], 0, 0],
                            [0, 0, g[2], 1 + g[3]],
                            [g[2], 1 + g[3], 1 + g[0], g[1]]])

    if m == 1:
        Beg_mat = np.array([[1, 0, 0, 0],
                            [0, 0, 0, 1],
                            [0, 1, 1, 0]])

    if m == 0:
        Beg_mat = np.array([[1/(1+g[0]), 0, 0, 0],
                            [0, 0, 0, 1 / (1+g[3])],
                            [0, 1/(1+g[1]+g[2]),1/(1+g[1]+g[2]), 0]])


    if m == -1:
        Beg_mat = np.array([[ (1 + g[0])**-2,                   0,                   0,              0],
                            [              0,                   0,                   0, (1 + g[3])**-2],
                            [              0, 1*(1+g[1]+g[2])**-2, 1*(1+g[1]+g[2])**-2,             0]])

    if m == -2:
        Beg_mat = np.array([[(1 + g[0]) ** -3, 0, 0, 0],
                            [0, 0, 0, (1 + g[3]) ** -3],
                            [0, (1 + g[1] + g[2]) ** -3, (1 + g[1] + g[2]) ** -3, 0]])

    Beg = Beg_mat.astype(float)

    return Beg


def get_material_stiffness_matrix_trad(ex, ey, dispVec, C, m):
    """
    Compute the material stiffness matrix for traditional seth-hill strain, K3
    """
    B_gv = tri3.get_Disp_grad(ex, ey)
    B_eg = get_B_eg_matrix_trad(ex, ey, dispVec, m)

    K3 = B_gv.T @ B_eg.T @ C @ B_eg @ B_gv
    return K3


def get_S_matrix(ex, ey, dispVec, C, m):
    """
    Compute the S-matrix for green strain.
    Hver kolonne i s matrisen er et produkt av B_eg.T*s / g
    """

    eps = tri3.get_strainVec_trad(ex, ey, dispVec, m)
    g = np.zeros(4)
    g = tri3.get_Disp_grad_Vec(ex, ey, dispVec)

    s = C @ eps
    if m == 2:
        B_eg1 = np.array([
            [1, 0, 0],
            [0, 0, 0],
            [0, 0, 1],
            [0, 0, 0]])

        B_eg2 = np.array([
            [0, 0, 0],
            [1, 0, 0],
            [0, 0, 0],
            [0, 0, 1]])

        B_eg3 = np.array([
            [0, 0, 1],
            [0, 0, 0],
            [0, 1, 0],
            [0, 0, 0]])

        B_eg4 = np.array([
            [0, 0, 0],
            [0, 0, 1],
            [0, 0, 0],
            [0, 1, 0]])

    if m == 1:
        B_eg1 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg2 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg3 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg4 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

    if m == 0:
        B_eg1 = np.array([
            [-1/(1+g[0])**2, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg2 = np.array([
            [0, 0, 0],
            [0, 0, 1/(1+g[1]+g[2])],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg3 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 1/(1+g[1]+g[2])],
            [0, 0, 0]])

        B_eg4 = np.array([
            [0, 0, 0],
            [0, 0, 1],
            [0, 0, 0],
            [0,-1/(1+g[3])**2, 0]])

    if m == -1:
        B_eg1 = np.array([
            [-2 * (1 + g[0]) ** -3, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg2 = np.array([
            [0, 0, 0],
            [0, 0, -2 * (1 + g[1] + g[2]) ** -3],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg3 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, -2 * (1 + g[1] + g[2]) ** -3],
            [0, 0, 0]])

        B_eg4 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, -2 * (1 + g[3]) ** -3, 0]])

    if m == -2:
        B_eg1 = np.array([
            [-3 * (1 + g[0]) ** -4, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg2 = np.array([
            [0, 0, 0],
            [0, 0, -3 * (1 + g[1] + g[2]) ** -4],
            [0, 0, 0],
            [0, 0, 0]])

        B_eg3 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, -3 * (1 + g[1] + g[2]) ** -4],
            [0, 0, 0]])

        B_eg4 = np.array([
            [0, 0, 0],
            [0, 0, 0],
            [0, 0, 0],
            [0, -3 * (1 + g[3]) ** -4, 0]])

    S1 = B_eg1 @ s
    S2 = B_eg2 @ s
    S3 = B_eg3 @ s
    S4 = B_eg4 @ s

    S = np.array([[S1[0], S2[0], S3[0], S4[0]],
                  [S1[1], S2[1], S3[1], S4[1]],
                  [S1[2], S2[2], S3[2], S4[2]],
                  [S1[3], S2[3], S3[3], S4[3]]])

    Smat = S.astype(float)
    return Smat


def get_geometric_stiffness_matrix_trad(ex, ey, dispVec, C, m):
    """
    Geometric stiffness matrix for traditional strain
    """
    B_gv = tri3.get_Disp_grad(ex, ey)

    S = get_S_matrix(ex, ey, dispVec, C, m)

    Kg = B_gv.T @ S @ B_gv
    K2 = Kg.astype(float)

    return K2

def get_tangent_Stiffness_matrix_trad(ex, ey, dispVec, C, m):
    """ Get tangent stiffness matrix for traditional formulation"""
    K2 = get_geometric_stiffness_matrix_trad(ex, ey, dispVec, C, m)
    K3 = get_material_stiffness_matrix_trad(ex, ey, dispVec, C, m)

    Kt = K2 + K3

    return Kt
