import numpy as np

def extract_set(mesh, setname, connectivity=True):
    '''
    Returns element connecivity array of given set
    Array data = node IDs
    If connectivity = False, returns list of node IDs stripped of duplicates

    :param mesh:            MeshIO object generated from .msh file
    :param setname:         string, setname as defined in Gmsh e.g. 'domain'
    :param connectivity:    boolean, if True gives connectivity array
    :return:                np.array or list, nodal IDs in given set
    '''

    loc = None

    for i in range(len(mesh.cell_sets[setname])):
        if mesh.cell_sets[setname][i].size:
            loc = i
            print('Found cells (elements) with setname =', setname, 'at location', loc)

    if loc is not None:
        if connectivity:
            return mesh.cells[loc].data
        else:
            return list(set(mesh.cells[loc].data.flatten()))
    else:
        print('Found no cells (elements) with setname =', setname)
        raise LookupError


def set_fixed_dofs(mesh, setname, k_sys, dofs=(0, 1, 2)):
    '''
    Sets constrains in dense matrix of K_sys

    :param mesh:    MeshIO object of mesh generated by Gmsh
    :param setname: string, name of the constraint set as defined in Gmsh e.g. 'fixedEnd'
    :param k_sys:   np.array (dofs, dofs)
    :param dofs:    tuple, components to be contrained for all nodes in set
    :return:        np.array (dofs, dofs), constrained system striffness matrix K_sys
    '''
    ndofs_per_node = 3  # endret fra 6 til 2, kun to frihetsgrader 
    nodes = extract_set(setname=setname, mesh=mesh, connectivity=False)
    for node in nodes:
        for dof in dofs:
            indx = int((node * ndofs_per_node) + dof)
            k_sys[indx, :] = 0.0
            k_sys[:, indx] = 0.0
            k_sys[indx, indx] = 1.0

    return k_sys


def add_load(mesh, setname, load_vec, load_magnitude, dof):
    '''
    Add load to an existing load_vector by lumping uniformly over given set of nodes

    :param mesh:            MeshIO object of mesh generated by Gmsh
    :param setname:         string, name of the load set as defined in Gmsh e.g. 'freeEnd'
    :param load_vec:        np.array (dofs,), existing load vector
    :param load_magnitude:  float, total load to be distributed among nodes in set
    :param dof:             integer, load direction
    :return:                np.array (dofs,), new load vector
    '''
    ndofs_per_node = 3  # endret fra 6 til 2
    nodes = extract_set(setname=setname, mesh=mesh, connectivity=False)
    nodal_load = load_magnitude / nodes.__len__()
    for node in nodes:
        indx = int((node * ndofs_per_node) + dof)
        load_vec[indx] += nodal_load

    return load_vec


def extract_element_coords(mesh, elements, indx):
    '''
    Retrives the element coordinates in form np.array(4,) as used by other functions

    :param mesh:        MeshIO object of mesh generated by Gmsh
    :param elements:    np.array(n, 4), connectivity array of elements
    :param indx:        integer, which element in the connectivity array to return the coordinates of
    :return:            tuple (np.array (4,), np.array (4,), np.array (4,)), element global coordinates
    '''
    elem = elements[indx]
    i, j, k, l = elem[0], elem[1], elem[2], elem[3]
    ecoords = np.array([mesh.points[i], mesh.points[j], mesh.points[k], mesh.points[l]])
    ex = ecoords[0:, 0]
    ey = ecoords[0:, 1]
    ez = ecoords[0:, 2]

    return ex, ey, ez


def extract_nodal_coords(mesh, nodeID):
    '''
    Helper function to extract nodal coordinates given a mesh and a nodeID

    :param mesh:    MeshIO object of mesh generated by Gmsh
    :param nodeID:  integer, node to retrieve
    :return:        tuple (x,y,z), global coordinates
    '''
    ncoords = mesh.points[nodeID]
    x = ncoords[0]
    y = ncoords[1]
    z = ncoords[2]

    return x, y, z


def extract_dof_indx(elements, indx):
    '''
    Helper function to extract the global dof indexes given an element connectivity array

    :param elements:    np.array(n, 4), connectivity array of elements
    :param indx:        integer, which element in the connectivity array to return the global dof indexes of
    :return:
    '''
    elem = elements[indx]
    i, j, k, l = elem[0], elem[1], elem[2], elem[3]
    
    #ENDRET
    dof_indx = np.array([
        i * 3, i * 3 + 1, i * 3 + 2,
        j * 3, j * 3 + 1, j * 3 + 2,
        k * 3, k * 3 + 1, k * 3 + 2, 
        l * 3, l * 3 + 1, l * 3 + 2,
    ])

    return dof_indx


def nodal_IDs_to_dof_indx(nodeset, dofs):
    '''
    Helper function to find the global dof indexes of a given node-set

    :param nodeset:     list, the nodeIDs
    :param dofs:        tuple, the components to be included
    :return:            list, the global dof indexes of given nodes and components
    '''
    dofset = []

    for node in nodeset:
        for dof in dofs:
            dofset.append(node * 3 + dof) # endret fra 6 til 3

    return dofset



def timoshenko_cantilever_end_load_displacement(L, B, H, E, P, kappa, nu):
    '''
    Calculates the tip displacement of a cantilever beam according to Timoshenko beam theory

    :param L:       float, length of beam
    :param B:       float, width of beam
    :param H:       float, height of beam
    :param E:       float, E-modulus
    :param P:       float, applied point load at end of beam
    :param kappa:   float, shear contribution factor (usually 5/6)
    :param nu:      float, Poisson
    :return:        float, tip displacement
    '''
    I = B * H ** 3 * 1 / 12
    G = E / 2 * (1 + nu)
    return ((P * L) / (kappa * B * H * G) + (P * L ** 3) / (E * I * 3))


def bernoulli_cantilever_end_load_displacement(L, B, H, E, P):
    '''
    Calculates the tip displacement of a cantilever beam according to Bernoulli beam theory

    :param L:       float, length of beam
    :param B:       float, width of beam
    :param H:       float, height of beam
    :param E:       float, E-modulus
    :param P:       float, applied point load at end of beam
    :return:        float, tip displacement
    '''
    I = B * H ** 3 * 1 / 12
    return (P * (L ** 3) / (3 * E * I))
